@@ -8,7 +8,7 @@ use Net::DPAP::Client::Image;
 use LWP::UserAgent;
 use URI;
 use base qw(Class::Accessor::Fast);
-our $VERSION = '0.24';
+our $VERSION = '0.25';
 
 __PACKAGE__->mk_accessors(qw(hostname port ua server databases_count
 item_name login_required dmap_protocol_version dpap_protocol_version
@@ -23,7 +23,7 @@ sub new {
 
   # Let's look like an iPhoto client
   my $ua = LWP::UserAgent->new(keep_alive => 1);
-  $ua->agent("iPhoto/4.01 (Macintosh; PPC)");
+  $ua->agent('iPhoto/4.01 (Macintosh; PPC)');
   $ua->default_headers->push_header('Client-DMAP-Version', '1.0');
   $ua->default_headers->push_header('Client-DPAP-Version', '1.0');
   $self->ua($ua);
@@ -114,8 +114,7 @@ sub connect {
     my $albumid = $album->id;
     my @images;
 
-    my $uri = URI->new("http://www.example.com:8770/databases/1/containers/$albumid/items?meta=dpap.aspectratio,dpap.imagefilesize,dpap.creationdate&type=photo");
-    $response = $self->do_get($uri);
+    $response = $self->do_get("databases/1/containers/$albumid/items", meta => 'dpap.aspectratio,dpap.imagefilesize,dpap.creationdate', type => 'photo');
     $dmap = dmap_unpack($response->content);
 
     assert($dmap->[0]->[0] eq 'daap.playlistsongs');
@@ -128,6 +127,10 @@ sub connect {
       foreach my $subtuple (@$value) {
 	assert($subtuple->[0] eq 'dmap.listingitem');
 	my $image = Net::DPAP::Client::Image->new();
+
+	my $ua = $self->ua;
+	$image->ua($ua);
+
 	foreach my $subsubtuple (@{$subtuple->[1]}) {
 	  my $subsubkey = $subsubtuple->[0];
 	  my $subsubvalue = $subsubtuple->[1];
@@ -137,10 +140,10 @@ sub connect {
 
 	my $imageid = $image->id;
 
-	my $thumbnail_url = $self->construct_uri("http://www.example.com:8770/databases/1/items?meta=dpap.thumb&query=('dmap.itemid:$imageid')");
+	my $thumbnail_url = $self->construct_uri('databases/1/items', meta => 'dpap.thumb', query => "('dmap.itemid:$imageid')");
 	$image->thumbnail_url($thumbnail_url);
 
-	my $hires_url = $self->construct_uri("http://www.example.com:8770/databases/1/items?meta=dpap.hires&query=('dmap.itemid:$imageid')");
+	my $hires_url = $self->construct_uri('databases/1/items', meta => 'dpap.hires', query => "('dmap.itemid:$imageid')");
 	$image->hires_url($hires_url);
 
 	push @images, $image;
@@ -155,9 +158,10 @@ sub connect {
 
 sub do_get {
   my $self = shift;
-  my $path = shift;
+  my ($path, @form) = @_;
+
   my $ua = $self->ua;
-  my $uri = $self->construct_uri($path);
+  my $uri = $self->construct_uri($path, @form);
 
   my $response = $ua->get($uri);
   die "Error when fetching $uri" unless $response->is_success;
@@ -165,28 +169,31 @@ sub do_get {
   return $response;
 }
 
+# Using URI module for URI parsing & constructing is more hassle than simply
+# storing & passing URI components separately
 sub construct_uri {
   my $self = shift;
-  my $path = shift;
-  my $uri;
-  if ($path =~ /http/) {
-    $uri = URI->new($path);
-  } else {
-    $uri = URI->new("http://www.foo.com/");
-    $uri->path($path);
-  }
-
-  my $hostname = $self->hostname;
+  my ($path, @form) = @_;
+  
+  my $host = $self->hostname;
   my $port = $self->port;
+ 
+  my $uri = "http://$host:$port/$path";
+ 
   my $session_id = $self->session_id;
+  if (defined $session_id) {
+    unshift @form, 'session-id' => $session_id;
+  }
+ 
+  if ($#form > 0) {
+    my ($key, $value, @form) = @form;
+    $uri .= "?$key=$value";
 
-  $uri->host($hostname);
-
-  $uri->port($port);
-  my %form = $uri->query_form;
-  $form{"session-id"} = $session_id if $session_id;
-  $uri->query_form(%form) if $session_id;
-  $uri =~ s/%3A/:/; # mmm, non-standard-compliant
+    while ($#form > 0) {
+      ($key, $value, @form) = @form;
+      $uri .= "&$key=$value";
+    }
+  }
   return $uri;
 }
 
